@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016  B.A.T.M.A.N. contributors:
+ * Copyright (C) 2012-2017  B.A.T.M.A.N. contributors:
  *
  * Simon Wunderlich
  *
@@ -258,7 +258,7 @@ static int unix_sock_req_data(struct globals *globals,
 	head->client_socket = client_sock;
 	head->requested_type = request->requested_type;
 
-	send_alfred_packet(interface, &globals->best_server->address,
+	send_alfred_packet(globals, interface, &globals->best_server->address,
 			   request, sizeof(*request));
 
 	return 0;
